package	jp.co.sds_corp.syslog.viewer.statistics; 

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.text.DecimalFormat;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import jp.co.sds_corp.nanchart.graph.Graph2D;
import jp.co.sds_corp.nanchart.graph.GraphStorage;
import jp.co.sds_corp.nanchart.graph.LineGraph;
import jp.co.sds_corp.nanchart.graph.XScale;
import jp.co.sds_corp.nanchart.graph.YScale;
import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.syslog.viewer.SyslogParam;
import jp.co.sds_corp.syslog.viewer.SyslogTable;
import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.syslog.SyslogMsg;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* 全般的状況の表示パネルです。
*/
public class GraphPanel extends JPanel
{
	private static final long serialVersionUID = 1L;
	Window		owner;
	SyslogParam		param;
	Vector<SyslogEvent>	vecLog;
	SyslogTable	table;
	ResourceBundle res;
	JButton BTN_UPD = new JButton( "Update" );
	JButton BTN_SETUP = new JButton( "Setup" );
	GraphSetupDialog	setup;
	JLabel		LBL_ALL;
	JTextField	TXT_ALL = new JTextField();
	JLabel		LBL_DISP;
	JTextField	TXT_DISP = new JTextField();
	Graph GRAPH = new Graph();
	ColorDefine	coldef = new ColorDefine();

	public GraphPanel( JDialog owner, SyslogParam param,
							Vector<SyslogEvent> vecLog, SyslogTable table )
	{
		this.owner = owner;
		this.param = param;
		this.vecLog = vecLog;
		this.table = table;
		setup = new GraphSetupDialog( owner, param );
		createControls();
	}

	public GraphPanel( JFrame owner, SyslogParam param,
							Vector<SyslogEvent> vecLog, SyslogTable table )
	{
		this.owner = owner;
		this.param = param;
		this.vecLog = vecLog;
		this.table = table;
		setup = new GraphSetupDialog( owner, param );
		createControls();
	}

	public GraphPanel( SyslogParam param,
							Vector<SyslogEvent> vecLog, SyslogTable table )
	{
		this.param = param;
		this.vecLog = vecLog;
		this.table = table;
		createControls();
	}

	protected void createControls() {
		res = ResourceBundle.getBundle( "resource.viewer" );
		String text;
		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );

		text = res.getString( "statistics.lbl1.text" );
		Box box1 = new Box( BoxLayout.X_AXIS );
		LBL_ALL = new JLabel( text );
		box1.add( LBL_ALL );
		TXT_ALL.setEditable( false );
		box1.add( TXT_ALL );
		box1.add( Box.createHorizontalStrut( 10 ) );

		text = res.getString( "statistics.lbl2.text" );
		LBL_DISP = new JLabel( text );
		box1.add( LBL_DISP );
		TXT_DISP.setEditable( false );
		box1.add( TXT_DISP );
		box1.add( Box.createHorizontalStrut( 10 ) );

		BTN_UPD.setMnemonic( KeyEvent.VK_ESCAPE );
		BTN_UPD.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				updateGraph();
			}
		});
		box1.add( BTN_UPD );
		BTN_SETUP.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				setup();
			}
		});
		box1.add( BTN_SETUP );
		add( box1 );

		add( GRAPH );
		setup.BTN_OK.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				setup.onOk();
/*
				if ( setup.getAutoUpdate()) {
					BTN_UPD.setEnabled( false );
				} else {
					BTN_UPD.setEnabled( true );
				}
*/
				updateGraph();
			}
		});
		updateGraph();
		updateText();
	}

	protected void received() {
		updateText();
		if ( setup.getAutoUpdate()) {
			updateGraph();
		}
	}
	protected void updateGraph() {
		calcGraph();
		repaint();
	}

	protected void updateText() {
		TXT_ALL.setText( String.valueOf( vecLog.size()));
		TXT_DISP.setText( String.valueOf( table.getModel().getRowCount()));
	}

	protected void setup() {
		WindowUtility.center( this, setup );
		setup.setVisible( true );
	}

	class Graph extends JComponent {
		private static final long serialVersionUID = 1L;
		Dimension dim = new Dimension( 500, 500 );

		public Dimension getPreferredSize() {
			return dim;
		}

		public void paint( Graphics gr ) {
			drawGraph( this, gr );
		}
	};

	TreeMap<Object,Map<String,Integer>> mapAll = new TreeMap<Object,Map<String,Integer>>();
	DecimalFormat df1 = new DecimalFormat( "00" );
	CmnDate to;
	CmnDate fr;
	int nMax;

	protected void calcGraph() {
		mapAll.clear();
		to = new CmnDate();
		fr = (CmnDate )to.clone();
		switch ( setup.getTerm()) {
		case GraphSetupDialog.TERM_6HOUR:
			fr.add( CmnDate.HOUR, -6 );
			break;
		case GraphSetupDialog.TERM_12HOUR:
			fr.add( CmnDate.HOUR, -12 );
			break;
		case GraphSetupDialog.TERM_24HOUR:
			fr.add( CmnDate.HOUR, -24 );
			break;
		case GraphSetupDialog.TERM_48HOUR:
			fr.add( CmnDate.HOUR, -48 );
			break;
		}

		for ( int i = 0; i < vecLog.size(); i++ ) {
			SyslogMsg msg = ((SyslogEvent )vecLog.get( i )).getMessage();
			CmnDate dt = msg.getDate();
			Object grp;
			switch ( setup.getGroup()) {
			case GraphSetupDialog.GRP_FACILITY:
				grp = msg.getFacility();
				break;
			case GraphSetupDialog.GRP_PRIORITY:
			default:
				grp = msg.getPriority();
				break;
			}
			Map<String,Integer> map = mapAll.get( grp );
			if ( map == null ) {
				map = new TreeMap<String,Integer>();
				mapAll.put( grp, map );
			}

			if ( dt.before( fr )) {
				continue;
			}
			String key;
			switch ( setup.getInterval()) {
			case GraphSetupDialog.INT_15MIN:
				key = dt.format( "yyyyMMddHH" )
					+ df1.format( ( dt.get( CmnDate.MINUTE ) / 15 ) * 15 );
				break;
			case GraphSetupDialog.INT_30MIN:
				key = dt.format( "yyyyMMddHH" )
					+ df1.format( ( dt.get( CmnDate.MINUTE ) / 30 ) * 30 );
				break;
			case GraphSetupDialog.INT_1HOUR:
				key = dt.format( "yyyyMMddHH" ) + "00";
				break;
			case GraphSetupDialog.INT_5MIN:
			default:
				key = dt.format( "yyyyMMddHH" )
					+ df1.format( ( dt.get( CmnDate.MINUTE ) / 5 ) * 5 );
				break;
			}
			Integer num = (Integer )map.get( key );
			if ( num == null ) {
				num = Integer.valueOf( 1 );
			} else {
				num = Integer.valueOf( num.intValue() + 1 );
			}
			map.put( key, num );
		}

		nMax = 0;
		Iterator<Object> itr = mapAll.keySet().iterator();
		while ( itr.hasNext()) {
			Object key = itr.next();
			Map<String,Integer> map = (Map<String,Integer> )mapAll.get( key );

			Iterator<String> itr2 = map.keySet().iterator();
			while ( itr2.hasNext()) {
				Object key2 = itr2.next();
				Integer num = (Integer )map.get( key2 );
				if ( nMax < num.intValue()) {
					nMax = num.intValue();
				}
			}
		}
	}

	protected void drawGraph( JComponent c, Graphics gr ) {
		Graph2D grf = new Graph2D( fr, to, Integer.valueOf(0), Integer.valueOf( nMax ));

		int nPos = 0;
		Iterator<Object> itr = mapAll.keySet().iterator();
		while ( itr.hasNext()) {
			Object key = itr.next();
			Map<String,Integer> map = (Map<String,Integer> )mapAll.get( key );

			LineGraph line = new LineGraph( fr, to, Integer.valueOf(0), Integer.valueOf( nMax ));
			line.setStroke(
				new BasicStroke( 1, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL ) );
			Color col = coldef.getColor( key );
			line.setLineColor( col );
			GraphStorage store = line.getStorage();
			grf.addSeries( line );

			Integer	ZERO = Integer.valueOf( 0 );
			CmnDate tmp = (CmnDate )fr.clone();
			for ( ;; ) {
				String key2;
				switch ( setup.getInterval()) {
				case GraphSetupDialog.INT_15MIN:
					key2 = tmp.format( "yyyyMMddHH" )
						+ df1.format( ( tmp.get( CmnDate.MINUTE ) / 15 ) * 15 );
					break;
				case GraphSetupDialog.INT_30MIN:
					key2 = tmp.format( "yyyyMMddHH" )
						+ df1.format( ( tmp.get( CmnDate.MINUTE ) / 30 ) * 30 );
					break;
				case GraphSetupDialog.INT_1HOUR:
					key2 = tmp.format( "yyyyMMddHH" ) + "00";
					break;
				case GraphSetupDialog.INT_5MIN:
				default:
					key2 = tmp.format( "yyyyMMddHH" )
						+ df1.format( ( tmp.get( CmnDate.MINUTE ) / 5 ) * 5 );
					break;
				}
				Integer num = (Integer )map.get( key2 );
				if ( num == null ) {
					num = ZERO;
				}
				try {
					CmnDate dt = new CmnDate( key2, "yyyyMMddHHmm" );
					store.add( dt, num );
				} catch ( Exception ex ) {
					ex.printStackTrace();
				}
				switch ( setup.getInterval()) {
				case GraphSetupDialog.INT_5MIN:
					tmp.add( CmnDate.MINUTE, 5 );
					break;
				case GraphSetupDialog.INT_15MIN:
					tmp.add( CmnDate.MINUTE, 15 );
					break;
				case GraphSetupDialog.INT_30MIN:
					tmp.add( CmnDate.MINUTE, 30 );
					break;
				case GraphSetupDialog.INT_1HOUR:
					tmp.add( CmnDate.HOUR, 1 );
					break;
				}
				if ( tmp.after( to )) {
					break;
				}
			}
			nPos++;
		}
		XScale  xs = (XScale )grf.getXScale();
		setXScale( xs, fr, to );
		YScale  ys = (YScale )grf.getYScale();
		setYScale( ys, 0, nMax );

		Rectangle rect = new Rectangle( 35, 5,
							(int )c.getWidth() - 50,
							(int )c.getHeight() - 20 );
		grf.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
							RenderingHints.VALUE_ANTIALIAS_ON);
		grf.drawGraph( (Graphics2D )gr, rect );
	}

	protected void setXScale( XScale xs, CmnDate fr, CmnDate to ) {
		Font font = new Font( "Dialog", Font.PLAIN, 8 );
		xs.setFont( font );
		CmnDate tmp = (CmnDate )fr.clone();
		tmp.set( CmnDate.SECOND, 0 );
		tmp.set( CmnDate.MINUTE, 0 );
		for ( ;; ) {
			tmp.add( CmnDate.HOUR, 1 );
			if ( tmp.after( to ) ) {
				break;
			}
			if ( setup.getTerm() == GraphSetupDialog.TERM_48HOUR ) {
				if ( tmp.get( CmnDate.HOUR ) % 6 != 0 ) {
					continue;
				}
			}
			xs.add( tmp.clone(), tmp.format( "HH" ));
		}
		xs.setScaleLine( true );
		xs.setLineColor( Color.gray );
	}

	protected void setYScale( YScale ys, int nMin, int nMax )
	{
		if ( nMax <= 0 ) {
			return;
		}
		Font font = new Font( "Dialog", Font.PLAIN, 8 );
		ys.setFont( font );
		String sc;
		sc = String.valueOf( nMin );
		ys.add( nMin, sc );
		sc = String.valueOf( nMax );
		ys.add( nMax, sc );

		int nDiv = (int )Math.pow( 10, sc.length() - 1 );
		if ( nDiv != 1 && ( nMax / nDiv ) < 5 ) {
			nDiv /= 2;
		}
		for ( int i = nMin; i <= nMax; i += nDiv ) {
			sc = String.valueOf( i );
			ys.add( i, sc );
		}
		ys.setScaleLine( true );
		ys.setLineColor( Color.gray );
	}

	public void updateUI() {
		super.updateUI();
		if ( setup != null ){
			SwingUtilities.updateComponentTreeUI( setup );
		}
	}

}
